@@ -31,6 +31,8 @@ use uucore::sum::{Digest, Sha3_224, Sha3_256, Sha3_384, Sha3_512, Shake128, Shak
 use uucore::translate;
 
 const NAME: &str = "hashsum";
+// Using the same read buffer size as GNU
+const READ_BUFFER_SIZE: usize = 32 * 1024;
 
 struct Options<'a> {
     algoname: &'static str,
@@ -97,8 +99,10 @@ fn create_algorithm_from_flags(matches: &ArgMatches) -> UResult<HashAlgorithm> {
         set_or_err(detect_algo("b3sum", None)?)?;
     }
     if matches.get_flag("sha3") {
-        let bits = matches.get_one::<usize>("bits").copied();
-        set_or_err(create_sha3(bits)?)?;
+        match matches.get_one::<usize>("bits") {
+            Some(bits) => set_or_err(create_sha3(*bits)?)?,
+            None => return Err(ChecksumError::BitsRequiredForSha3.into()),
+        }
     }
     if matches.get_flag("sha3-224") {
         set_or_err(HashAlgorithm {
@@ -496,7 +500,7 @@ pub fn uu_app_custom() -> Command {
 /// hashsum is handled differently in build.rs
 /// therefore, this is different from other utilities.
 fn uu_app(binary_name: &str) -> (Command, bool) {
-    match binary_name {
+    let (command, is_hashsum_bin) = match binary_name {
         // These all support the same options.
         "md5sum" | "sha1sum" | "sha224sum" | "sha256sum" | "sha384sum" | "sha512sum" => {
             (uu_app_common(), false)
@@ -514,7 +518,19 @@ fn uu_app(binary_name: &str) -> (Command, bool) {
         "b3sum" => (uu_app_b3sum(), false),
         // We're probably just being called as `hashsum`, so give them everything.
         _ => (uu_app_custom(), true),
-    }
+    };
+
+    // If not called as generic hashsum, override the command name and usage
+    let command = if is_hashsum_bin {
+        command
+    } else {
+        let usage = translate!("hashsum-usage-specific", "utility_name" => binary_name);
+        command
+            .help_template(uucore::localized_help_template(binary_name))
+            .override_usage(format_usage(&usage))
+    };
+
+    (command, is_hashsum_bin)
 }
 
 #[allow(clippy::cognitive_complexity)]
@@ -527,23 +543,26 @@ where
     for filename in files {
         let filename = Path::new(filename);
 
-        let mut file = BufReader::new(if filename == OsStr::new("-") {
-            Box::new(stdin()) as Box<dyn Read>
-        } else {
-            let file_buf = match File::open(filename) {
-                Ok(f) => f,
-                Err(e) => {
-                    eprintln!(
-                        "{}: {}: {e}",
-                        options.binary_name,
-                        filename.to_string_lossy()
-                    );
-                    err_found = Some(ChecksumError::Io(e));
-                    continue;
-                }
-            };
-            Box::new(file_buf) as Box<dyn Read>
-        });
+        let mut file = BufReader::with_capacity(
+            READ_BUFFER_SIZE,
+            if filename == OsStr::new("-") {
+                Box::new(stdin()) as Box<dyn Read>
+            } else {
+                let file_buf = match File::open(filename) {
+                    Ok(f) => f,
+                    Err(e) => {
+                        eprintln!(
+                            "{}: {}: {e}",
+                            options.binary_name,
+                            filename.to_string_lossy()
+                        );
+                        err_found = Some(ChecksumError::Io(e));
+                        continue;
+                    }
+                };
+                Box::new(file_buf) as Box<dyn Read>
+            },
+        );
 
         let (sum, _) = digest_reader(
             &mut options.digest,
