@@ -615,3 +615,28 @@ fn test_chgrp_non_utf8_paths() {
 
     ucmd.arg(current_gid.to_string()).arg(&filename).succeeds();
 }
+
+#[test]
+fn test_chgrp_recursive_on_file() {
+    // Test for regression where `chgrp -R` on a regular file would fail
+    // with "Not a directory" error. This should succeed since there's nothing
+    // to recurse into, similar to GNU chgrp behavior.
+    // equivalent of tests/chgrp/recurse in GNU coreutils
+    use std::os::unix::fs::MetadataExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch("regular_file");
+
+    let current_gid = getegid();
+
+    ucmd.arg("-R")
+        .arg(current_gid.to_string())
+        .arg("regular_file")
+        .succeeds()
+        .no_stderr();
+
+    assert_eq!(
+        at.plus("regular_file").metadata().unwrap().gid(),
+        current_gid
+    );
+}
