@@ -123,6 +123,46 @@ fn test_df_output() {
     assert_eq!(actual, expected);
 }
 
+#[test]
+fn test_df_follows_symlinks() {
+    let output = new_ucmd!()
+        .arg("-h")
+        .arg("--output=source")
+        .succeeds()
+        .stdout_str_lossy();
+
+    let filesystems = output.lines().skip(1).collect::<Vec<&str>>();
+    assert!(
+        filesystems
+            .iter()
+            .all(|&x| !std::path::Path::new(x).is_symlink())
+    );
+}
+
+#[test]
+fn test_df_trailing_zeros() {
+    use regex::Regex;
+
+    new_ucmd!()
+        .arg("-h")
+        .arg("--output=size,used")
+        .arg("--total")
+        .succeeds()
+        .stdout_does_not_match(&Regex::new("\\s[1-9][A-Z]").unwrap());
+}
+
+#[test]
+fn test_df_rounding() {
+    use regex::Regex;
+
+    new_ucmd!()
+        .arg("-H")
+        .arg("--output=size,used")
+        .arg("--total")
+        .succeeds()
+        .stdout_does_not_match(&Regex::new("\\s\\d{3}\\.\\d[A-Z]").unwrap());
+}
+
 #[test]
 fn test_df_output_overridden() {
     let expected = if cfg!(target_os = "macos") {
@@ -379,34 +419,36 @@ fn test_total() {
     //     ...
     //     /dev/loop14               63488    63488         0 100% /snap/core20/1361
     //     total                 258775268 98099712 148220200  40% -
-    let output = new_ucmd!().arg("--total").succeeds().stdout_str_lossy();
+    // Use explicit numeric-only columns to avoid whitespace issues in Filesystem or Mounted on columns
+    let output = new_ucmd!()
+        .arg("--output=size,used,avail")
+        .arg("--total")
+        .succeeds()
+        .stdout_str_lossy();
 
     // Skip the header line.
     let lines: Vec<&str> = output.lines().skip(1).collect();
 
-    // Parse the values from the last row.
+    // Parse the values from the last row (report totals)
     let last_line = lines.last().unwrap();
     let mut iter = last_line.split_whitespace();
-    assert_eq!(iter.next().unwrap(), "total");
-    let reported_total_size = iter.next().unwrap().parse().unwrap();
-    let reported_total_used = iter.next().unwrap().parse().unwrap();
-    let reported_total_avail = iter.next().unwrap().parse().unwrap();
+    let reported_total_size: u64 = iter.next().unwrap().parse().unwrap();
+    let reported_total_used: u64 = iter.next().unwrap().parse().unwrap();
+    let reported_total_avail: u64 = iter.next().unwrap().parse().unwrap();
 
     // Loop over each row except the last, computing the sum of each column.
-    let mut computed_total_size = 0;
-    let mut computed_total_used = 0;
-    let mut computed_total_avail = 0;
+    let mut computed_total_size: u64 = 0;
+    let mut computed_total_used: u64 = 0;
+    let mut computed_total_avail: u64 = 0;
     let n = lines.len();
     for line in &lines[..n - 1] {
         let mut iter = line.split_whitespace();
-        iter.next().unwrap();
         computed_total_size += iter.next().unwrap().parse::<u64>().unwrap();
         computed_total_used += iter.next().unwrap().parse::<u64>().unwrap();
         computed_total_avail += iter.next().unwrap().parse::<u64>().unwrap();
     }
 
-    // Check that the sum of each column matches the reported value in
-    // the last row.
+    // Check that the sum of each column matches the reported value in the last row.
     assert_eq!(computed_total_size, reported_total_size);
     assert_eq!(computed_total_used, reported_total_used);
     assert_eq!(computed_total_avail, reported_total_avail);
@@ -648,6 +690,24 @@ fn test_block_size_from_env() {
     assert_eq!(get_header("BLOCKSIZE", "333"), "333B-blocks");
 }
 
+#[test]
+fn test_block_size_from_env_zero() {
+    fn get_header(env_var: &str, env_value: &str) -> String {
+        let output = new_ucmd!()
+            .arg("--output=size")
+            .env(env_var, env_value)
+            .succeeds()
+            .stdout_str_lossy();
+        output.lines().next().unwrap().trim().to_string()
+    }
+
+    let default_block_size_header = "1K-blocks";
+
+    assert_eq!(get_header("DF_BLOCK_SIZE", "0"), default_block_size_header);
+    assert_eq!(get_header("BLOCK_SIZE", "0"), default_block_size_header);
+    assert_eq!(get_header("BLOCKSIZE", "0"), default_block_size_header);
+}
+
 #[test]
 fn test_block_size_from_env_precedences() {
     fn get_header(one: (&str, &str), two: (&str, &str)) -> String {
@@ -705,6 +765,16 @@ fn test_invalid_block_size_from_env() {
     let header = output.lines().next().unwrap().trim().to_string();
 
     assert_eq!(header, default_block_size_header);
+
+    let output = new_ucmd!()
+        .arg("--output=size")
+        .env("DF_BLOCK_SIZE", "0")
+        .env("BLOCK_SIZE", "222")
+        .succeeds()
+        .stdout_str_lossy();
+    let header = output.lines().next().unwrap().trim().to_string();
+
+    assert_eq!(header, default_block_size_header);
 }
 
 #[test]
